/* globals lInfo, lWarn, Property, HTTP, WebserviceConnection, unescape */
// 'use strict';

var httpCodes = {
    OK: 200
};
var cloudResponseCodes = {
    OK: 0,
    NO_RECEIVERS: 11
};

var localEndpoint = {
    sendNotification: function (uuids, text) {
        var targetAll = uuids === ALL_RECEIVERS;
        if (!uuids || !targetAll && !uuids.length) {
            return;
        }
        var that = this;
        var urlTemplate = targetAll
            ? 'http://localhost:8089/backend/v1/notifications/receivers/sendTextMessage'
            : 'http://localhost:8089/backend/v1/notifications/receivers/${receiverId}/sendTextMessage';
        var body = { text: this.unicodeEscape(unescape(text)) };
        var failures = [];
        lInfo.logln('makePostCalls to local endpoint:');
        if (targetAll) {
            try {
                var response = that.makePostCall(urlTemplate, '', JSON.stringify(body).split('\\\\').join('\\'));
                if (!response || !response.success || response.body && response.body.error) {
                    failures.push({ receiverId: ALL_RECEIVERS, body: body, response: response });
                }
            } catch (e) {
                failures.push({ receiverId: ALL_RECEIVERS, body: null, response: e });
            }
        } else {
            uuids.forEach(function (uuid) {
                try {
                    var response = that.makePostCall(urlTemplate.replace('${receiverId}', uuid), '', JSON.stringify(body).split('\\\\').join('\\'));
                    if (!response || !response.success || response.body && response.body.error) {
                        failures.push({ receiverId: uuid, body: body, response: response });
                    }
                } catch (e) {
                    failures.push({ receiverId: uuid, body: null, response: e });
                }
            });
        }
        if (failures.length) {
            failures.forEach(function (failure) {
                lInfo.logln('POST for ' + failure.receiverId + ' with body: ' + JSON.stringify(failure.body) + ' responded with: ' + JSON.stringify(failure.response));
            });
        } else {
            lInfo.logln('... OK');
        }
    },
    fetchNotificationListFromLocalEndpoint: function (receiversToAppendTo, errors, callback) {
        lInfo.logln('fetchNotificationListFromLocalEndpoint called');
        lInfo.logln('makeGetCall to local endpoint: http://localhost:8089/backend/v1/notifications/receivers');
        var response;
        var data;
        var allReceivers;
        try {
            response = this.makeGetCall('http://localhost:8089/backend/v1/notifications/receivers', 'include=notificationHubRegistrations');
            data = JSON.parse(response.body).data;
            lInfo.logln('... OK');
            var notificationHubRegistrations = data.included && data.included.notificationHubRegistrations || [];
            var newApiReceivers = data.receivers
            .filter(function (receiver) {
                return receiver.attributes && receiver.attributes.notificationHubRegistration != null;
            })
            .map(function (receiver) {
                var mobileName = receiver.attributes.deviceName || 'unknown device';
                var appName = 'unknown application';
                var notificationHubRegistrationDetails = notificationHubRegistrations.filter(function (notificationHubRegistration) {
                    return notificationHubRegistration.id === receiver.attributes.notificationHubRegistration;
                });
                if (notificationHubRegistrationDetails[0]) {
                    appName = notificationHubRegistrationDetails[0].attributes.mobileApplicationName;
                }
                return {
                    MobileName: mobileName,
                    MobileAppUUID: receiver.id,
                    AppName: appName,
                    localEndpoint: true
                };
            });
            allReceivers = newApiReceivers.concat(receiversToAppendTo);
            lInfo.logln('all receivers: ' + JSON.stringify(allReceivers));
        } catch (e) {
            lWarn.logln('... GET receivers failed');
            lWarn.logln(JSON.stringify(response) + ', error: ' + e);
            errors.push('List of receivers could not be read.');
        }
        callback(allReceivers || receiversToAppendTo, errors);
    },
    h: null,
    initiateHttp: function () {
        if (this.h) {
            return;
        }

        var extraheaders = [];
        extraheaders['User-Agent'] = 'dSS digitalStrom Server';

        this.h = new HTTP({
            cookieFile: '',
            username: '',
            password: '',
            authentication: 3,       // 1 = basic, 2 = digest
            noRedirect: 0,
            maxRedirects: 12,
            noPeerVerification: 1,   // don't bother about ssl peer verification
            debug: 0                 // enable console debug log
        },
        extraheaders);
    },
    unicodeEscape: function (str) {
        return str.replace(/[\s\S]/g, function (escape) {
            return '\\u' + ('0000' + escape.charCodeAt().toString(16)).slice(-4);
        });
    },
    makeGetCall: function (url, query) {
        this.initiateHttp();
        var response = this.h.get(url + (query ? '?' + query : ''));
        return response;
    },
    makePostCall: function (url, query, body) {
        this.initiateHttp();
        var response = this.h.post(url + (query ? '?' + query : ''), '', body);
        return response;
    }
};

function logRequestResult (code, resultData) {
    if (code === httpCodes.OK) {
        lInfo.logln(': OK [' + code + ']');
    } else {
        lWarn.logln(': FAILED! Server replied: "' + code + '"');
    }
    lInfo.logln(': body [' + resultData + ']');
}

var cloudService = {
    makePostCall: function (sUrl, queryString, data) {
        lInfo.logln('http post request ' + sUrl);
        lInfo.logln('  body ' + data);
        try {
            var headers = { 'Content-Type': 'application/json', 'NotificationTextIsUrlEncodedUtf7': 'true' };
            var handle = new WebserviceConnection();
            lInfo.logln('making new post ' + sUrl + ' with data ' + data);
            if (this.isNewWebrequest()) {
                handle.request(sUrl, queryString, 'POST', headers,
                data, true, logRequestResult);
            } else {
                handle.request(sUrl + '?' + queryString + '&token=' + this.getOSPToken(), 'POST', headers,
                data, logRequestResult);
            }
            return;
        } catch (e) {
            lWarn.logln('Exception during http post request ' + sUrl + ': ' + e);
        }
        return;
    },
    makeGetCall: function (url, queryString, callback) {
        lInfo.logln('makeGetCall to cloud endpoint: ' + url);

        function responseCallback (httpCode, response) {
            logRequestResult(httpCode, response);
            try {
                if (httpCode !== httpCodes.OK) {
                    throw new Error('Incorrect response - http code ' + httpCode);
                } else {
                    var body = JSON.parse(response);
                    var receivers = body.Response && body.Response.MobileApps || [];
                    var errors = body.ReturnCode != null && body.ReturnCode !== cloudResponseCodes.OK && body.ReturnCode !== cloudResponseCodes.NO_RECEIVERS
                    ? [ body.ReturnMessage ]
                    : [];
                    callback(receivers, errors);
                }
            } catch (e) {
                lWarn.logln('Error GETing ' + url + ': ' + e);
                callback([], [ 'List of receivers could not be read.' ]);
            }
        }

        try {
            var handle = new WebserviceConnection();
            if (this.isNewWebrequest()) {
                handle.simplerequest(url, queryString, 'GET', true, responseCallback);
            } else {
                handle.simplerequest(url + '?' + queryString + '&token=' + this.getOSPToken(), 'GET', responseCallback);
            }
        } catch (e) {
            lWarn.logln('Error GETting ' + url + ': ' + e);
            callback([], [ 'List of receivers could not be read.' ]);
        }
    },
    isNewWebrequest: function () {
        return Property.getNode('/scripts/system-addon-remote-connectivity') != null;
    },
    getOSPToken: function () {
        if (Property.getNode('/scripts/remote-connectivity/OSPToken') != null) {
            return Property.getNode('/scripts/remote-connectivity/OSPToken').getValue();
        }
        if (Property.getNode('/scripts/system-addon-remote-connectivity/OSPToken') != null) {
            return Property.getNode('/scripts/system-addon-remote-connectivity/OSPToken').getValue();
        }
        return null;
    },
    getCloudEmail: function () {
        if (Property.getNode('/scripts/remote-connectivity/username') != null) {
            return Property.getNode('/scripts/remote-connectivity/username').getValue().trim();
        }
        if (Property.getNode('/scripts/system-addon-remote-connectivity/username') != null) {
            return Property.getNode('/scripts/system-addon-remote-connectivity/username').getValue().trim();
        }
        return null;
    },
    getMyDSID: function () {
        return Property.getProperty('/system/dSID');
    },
    fetchNotificationList: function (func) {
        // GetMobileApps
        var url = 'public/accessmanagement/v1_0/RemoteConnectivity/GetMobileApps';
        var query = 'email=' + this.getCloudEmail();
        try {
            this.makeGetCall(url, query, function (response, errors) {
                localEndpoint.fetchNotificationListFromLocalEndpoint.call(localEndpoint, response, errors, func);
            });
        } catch (e) {
            lWarn.logln('Error: ' + e.toString());
            func([], [ e.toString() ]);
        }
    },
    sendNotification: function (uuida, text) {
        if (!uuida || !uuida.length) {
            return;
        }
        var url = 'public/notification/v1_0/SendNotification/SendPushNotification';
        var query = 'notificationID=PUSH_NOTIFICATION_DSS_EVENT_PUSH&languageCode=de-DE';
        var body = uuida
            .map(function (uuid) { return '"' + uuid + '"'; })
            .join(',');
        body = '{"MobileAppUUIDList": [' + body + '],"NotificationKeyValueList": { "Text": "' + text + '" } }';
        this.makePostCall(url, query, body);
    }
};
